'***********************************************************************
'*              Date/Time Routines for MS BASICs, v1.02
'*                           by Joe Negron
'*                          October 20, 1995
'***********************************************************************
'* MODULE Date
'*
'* PURPOSE
'*    Contains a number of date and time routines.
'*
'* NOTE
'*    Many of the routines make mention of a "standard date format",
'*    which is a date in the format "MM-DD-YYYY" (many of the routines
'*    will accept "MM-DD-YY", but I recommend the former).  Also,
'*    "standard time format" refers to a time in the "HH:MM:SS" format
'*    (although "HH:MM" is sometimes acceptable).
'*
'* EXTERNAL ROUTINE(S)
'*    QBX.LIB
'*       SUB Interrupt (IntNum%, InReg AS RegType, OutReg AS RegType)
'***********************************************************************
DEFINT A-Z

'$INCLUDE: 'qbx.bi'

'Date routines
DECLARE SUB Date2MDY (Month%, Day%, Year%, DOW$)

DECLARE FUNCTION Date2Day% (DateX$)
DECLARE FUNCTION Date2Eng$ (DateX$)
DECLARE FUNCTION Date2Mth% (DateX$)
DECLARE FUNCTION Date2Serial& (DateX$)
DECLARE FUNCTION Date2Year% (DateX$)
DECLARE FUNCTION DayOfTheCentury& (DateX$)
DECLARE FUNCTION DayOfTheWeek$ (DateX$)
DECLARE FUNCTION DayOfTheYear% (DateX$)
DECLARE FUNCTION DaysBetweenDates& (Date1$, Date2$)
DECLARE FUNCTION LeapYear% (Year%)
DECLARE FUNCTION MthName$ (DateX$)
DECLARE FUNCTION MDY2Date$ (Month%, Day%, Year%)
DECLARE FUNCTION Serial2Date$ (Serial&)
DECLARE FUNCTION ValidDate% (DateX$)
DECLARE FUNCTION WeekDay$ ()

'Time routines
DECLARE FUNCTION CalcTime$ (Time1$, Op%, Time2$)
DECLARE FUNCTION HMS2Time$ (Hour%, Minute%, Second%)
DECLARE FUNCTION Second2Date$ (Second&)
DECLARE FUNCTION Second2Time$ (Second&)
DECLARE FUNCTION Time2Hour% (TimeX$)
DECLARE FUNCTION Time2Meridian$ (TimeX$)
DECLARE FUNCTION Time2Minute% (TimeX$)
DECLARE FUNCTION Time2Second% (TimeX$)
DECLARE FUNCTION TimeDate2Second& (TimeX$, DateX$)

'Miscellaneous routines
DECLARE FUNCTION IntVal% (Number$)           'VAL() replacement

DIM SHARED Regs AS RegType

'***********************************************************************
'* FUNCTION CalcTime$
'*
'* PURPOSE
'*    Adds or subtracts Time1$ and Time2$ based on Operator$.
'*
'* EXAMPLE
'*    Time1$ = "03:00"
'*    Time2$ = "04:00"
'*    Op% = 1                 'Any value other than 0 is addition
'*    Elapsed$ = CalcTime$(Time1$, Op%, Time2$)
'*
'*    Elapsed$ is assigned "07:00"
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION CalcTime$ (Time1$, Op%, Time2$) STATIC
   Hour1% = IntVal%(Time1$)
   Hour2% = IntVal%(Time2$)
   Minute1% = IntVal%(MID$(Time1$, 4))
   Minute2% = IntVal%(MID$(Time2$, 4))
   Second1% = IntVal%(MID$(Time1$, 7))
   Second2% = IntVal%(MID$(Time2$, 7))

   IF Op% THEN                               'addition
      HourVal% = Hour1% + Hour2%
      MinVal% = Minute1% + Minute2%
      SecVal% = Second1% + Second2%

      IF SecVal% > 59 THEN                   'carry a minute?
         SecVal% = SecVal% - 60
         MinVal% = MinVal% + 1
      END IF

      IF MinVal% > 59 THEN                   'carry a hour?
         MinVal% = MinVal% - 60
         HourVal% = HourVal% + 1
      END IF

      DO WHILE HourVal% > 23                 'next day?
         HourVal% = HourVal% - 24
      LOOP
   ELSE                                      'subtraction
      IF Second2% > Second1% THEN            'borrow a minute?
         Second1% = Second1% + 60
         Minute1% = Minute1% - 1
      END IF

      SecVal% = Second1% - Second2%

      IF Minute2% > Minute1% THEN            'borrow an hour?
         Minute1% = Minute1% + 60
         Hour1% = Hour1% - 1
      END IF

      MinVal% = Minute1% - Minute2%

      IF Hour2% > Hour1% THEN                'borrow a day?
         DO UNTIL Hour1% > Hour2%
            Hour1% = Hour1% + 24
         LOOP
      END IF

      HourVal% = Hour1% - Hour2%
   END IF

   'set the final result
   Hour$ = MID$(STR$(HourVal%), 2)
   Minute$ = MID$(STR$(MinVal%), 2)
   Second$ = MID$(STR$(SecVal%), 2)

   DO WHILE LEN(Hour$) < 2                   'pad any empty slots
      Hour$ = "0" + Hour$
   LOOP

   DO WHILE LEN(Minute$) < 2
      Minute$ = "0" + Minute$
   LOOP

   DO WHILE LEN(Second$) < 2
      Second$ = "0" + Second$
   LOOP

   CalcTime$ = Hour$ + ":" + Minute$ + ":" + Second$

   Hour$ = ""
   Minute$ = ""
   Second$ = ""
END FUNCTION

'***********************************************************************
'* FUNCTION Date2Day%
'*
'* PURPOSE
'*    Returns the day number given a date in the standard date format.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    D% = Date2Day%(DateX$)
'*
'*    D% is assigned 20.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION Date2Day% (DateX$) STATIC
   Date2Day% = IntVal%(MID$(DateX$, 4, 2))
END FUNCTION

'***********************************************************************
'* FUNCTION Date2Eng$
'*
'* PURPOSE
'*    Returns a string variable representing the English form of the
'*    date given a date in the standard date format.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    D$ = Date2Eng$(DateX$)
'*
'*    D$ is assigned "20 Oct 95".
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION Date2Day% (DateX$)
'*    FUNCTION Date2Year% (DateX$)
'*    FUNCTION MthName$ (DateX$)
'***********************************************************************
FUNCTION Date2Eng$ (DateX$) STATIC
   Date2Eng$ = MID$(STR$(Date2Day%(DateX$)), 2) + " " + MthName$(DateX$) + " " + RIGHT$(STR$(Date2Year%(DateX$)), 2)
END FUNCTION

'***********************************************************************
'* SUB Date2MDY
'*
'* PURPOSE
'*    Uses DOS ISR 21h, Function 2Ah (Get Date) to return the month,
'*    day, year, and day of the week.
'*
'* EXAMPLE
'*    The date is Fri, 10-20-1995.
'*
'*    Date2MDY Month%, Day%, Year%, DOW$
'*
'*    The parameters are returned as:
'*       Month% = 10
'*       Day% = 20
'*       Year% = 1995
'*       DOW$ = "Fri"
'*
'* EXTERNAL ROUTINE(S)
'*    QBX.LIB
'*       SUB Interrupt (IntNum%, InReg AS RegType, OutReg AS RegType)
'***********************************************************************
SUB Date2MDY (Month%, Day%, Year%, DOW$) STATIC
   Regs.ax = &H2A00
   Interrupt &H21, Regs, Regs
   DOW% = Regs.ax AND &HFF                   'extract al register
   Month% = Regs.dx \ 256
   Day% = Regs.dx MOD 256
   Year% = Regs.cx
   DOW$ = MID$("SunMonTueWedThuFriSat", (DOW% + 1) * 3 - 2, 3)
END SUB

'***********************************************************************
'* FUNCTION Date2Mth%
'*
'* PURPOSE
'*    Returns the month number given a date in the standard date format.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    M% = Date2Mth%(DateX$)
'*
'*    M% is assigned 10.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION Date2Mth% (DateX$) STATIC
   Date2Mth% = IntVal%(LEFT$(DateX$, 2))
END FUNCTION

'***********************************************************************
'* FUNCTION Date2Serial&
'*
'* PURPOSE
'*    Returns the astronomical Julian day number given a date in the
'*    standard date format.  Note that the year must be 1583 or greater.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    Serial& = Date2Serial&(DateX$)
'*
'*    Serial& is assigned 2,450,011.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION Date2Day% (DateX$)
'*    FUNCTION Date2Mth% (DateX$)
'*    FUNCTION Date2Year% (DateX$)
'***********************************************************************
FUNCTION Date2Serial& (DateX$) STATIC
   Month% = Date2Mth%(DateX$)
   Day% = Date2Day%(DateX$)
   Year% = Date2Year%(DateX$)

   IF Month% > 2 THEN
      Month% = Month% - 3
   ELSE
      Month% = Month% + 9
      Year% = Year% - 1
   END IF

   TA& = 146097 * (Year% \ 100) \ 4
   TB& = 1461& * (Year% MOD 100) \ 4
   TC& = (153 * Month% + 2) \ 5 + Day% + 1721119
   Date2Serial& = TA& + TB& + TC&
END FUNCTION

'***********************************************************************
'* FUNCTION Date2Year%
'*
'* PURPOSE
'*    Returns the year number given a date in the standard date format.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    Y% = Date2Year%(DateX$)
'*
'*    Y% is assigned 1995 (or 95 if date is given in "MM-DD-YY" format).
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION Date2Year% (DateX$) STATIC
   Date2Year% = IntVal%(MID$(DateX$, 7))
END FUNCTION

'***********************************************************************
'* FUNCTION DayOfTheCentury&
'*
'* PURPOSE
'*    Returns the number of the day of the century given a date in the
'*    standard date format.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    D& = DayOfTheCentury&(DateX$)
'*
'*    D& is assigned 34,991.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION Date2Year% (DateX$)
'*    FUNCTION DaysBetweenDates& (Date1$, Date2$)
'*    FUNCTION MDY2Date$ (Month%, Day%, Year%)
'***********************************************************************
FUNCTION DayOfTheCentury& (DateX$) STATIC
   Year% = Date2Year%(DateX$)
   DayOfTheCentury& = DaysBetweenDates&(MDY2Date$(12, 31, Year% - (Year% MOD 100) - 1), DateX$)
END FUNCTION

'***********************************************************************
'* FUNCTION DayOfTheWeek$
'*
'* PURPOSE
'*    Returns a string stating the day of the week given a date in the
'*    standard date format.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    DayOfWk$ = DayOfTheWeek$(DateX$)
'*
'*    DayOfWk$ is assigned "Fri"
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION Date2Serial& (DateX$)
'***********************************************************************
FUNCTION DayOfTheWeek$ (DateX$) STATIC
   DayOfTheWeek$ = MID$("MonTueWedThuFriSatSun", ((Date2Serial&(DateX$) MOD 7) + 1) * 3 - 2, 3)
END FUNCTION

'***********************************************************************
'* FUNCTION DayOfTheYear%
'*
'* PURPOSE
'*    Returns the number of the day of the year (1-366) given a date in
'*    the standard date format.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    Y% = DayOfTheYear%(DateX$)
'*
'*    Y% is assigned 293.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION Date2Year% (DateX$)
'*    FUNCTION DaysBetweenDates& (Date1$, Date2$)
'*    FUNCTION MDY2Date$ (Month%, Day%, Year%)
'***********************************************************************
FUNCTION DayOfTheYear% (DateX$) STATIC
   DayOfTheYear% = DaysBetweenDates&(MDY2Date$(12, 31, Date2Year%(DateX$) - 1), DateX$)
END FUNCTION

'***********************************************************************
'* FUNCTION DaysBetweenDates&
'*
'* PURPOSE
'*    Returns the number of days between any two dates.  These two dates
'*    are to be given in the standard date format.
'*
'* EXAMPLE
'*    Date1$ = "01-01-1990"
'*    Date2$ = "10-20-1995"
'*    D& = DaysBetweenDates&(Date1$, Date2$)
'*
'*    D& is assigned 2,118.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION Date2Serial& (DateX$)
'***********************************************************************
FUNCTION DaysBetweenDates& (Date1$, Date2$) STATIC
   DaysBetweenDates& = ABS(Date2Serial&(Date1$) - Date2Serial&(Date2$))
END FUNCTION

'***********************************************************************
'* FUNCTION HMS2Time$
'*
'* PURPOSE
'*    Returns the time in the standard time format.
'*
'* EXAMPLE
'*    Hour% = 1
'*    Minute% = 2
'*    Second% = 0
'*    T$ = HMS2Time$(Hour%, Minute%, Second%)
'*
'*    T$ is assigned "01:02:00"
'***********************************************************************
FUNCTION HMS2Time$ (Hour%, Minute%, Second%) STATIC
   HMS2Time$ = RIGHT$("0" + MID$(STR$(Hour%), 2), 2) + ":" + RIGHT$("0" + MID$(STR$(Minute%), 2), 2) + ":" + RIGHT$("0" + MID$(STR$(Second%), 2), 2)
END FUNCTION

'***********************************************************************
'* FUNCTION IntVal%
'*
'* PURPOSE
'*    An integer-only VAL substitute that reduces .EXE size up to 10K.
'*
'* CREDIT(S)
'*    Dean Pidgeon (Enfield, NH)
'*
'*    Joe Negron (Fri, 20 Oct 95).  Modified to bail out when an invalid
'*    (non-numeric) character is encounted (for example, previously, the
'*    routine would return 900 for the string "09:00" it now returns 9).
'***********************************************************************
FUNCTION IntVal% (Number$) STATIC
   Length% = LEN(Number$)

   N% = 0
   FOR I% = 1 TO Length%
      C% = ASC(MID$(Number$, I%))
      IF C% > 47 AND C% < 58 THEN
         N% = N% * 10 + C% - 48
      ELSE
         EXIT FOR
      END IF
   NEXT I%

   IntVal% = N%
END FUNCTION

'***********************************************************************
'* FUNCTION LeapYear%
'*
'* PURPOSE
'*    Determines whether or not the given year is a leap year.
'*
'*    A year is a leap year every four years, except centenary years,
'*    except every fourth centenary year.  For example, 1992 is a leap
'*    year because it is evenly divisible by 4 and is not a centenary
'*    year; 1900 is not a leap year because it is a centenary year, but
'*    2000 is a leap year because it is a fourth (evenly divisible by
'*    400) centenary year.
'*
'* EXAMPLE
'*    Year% = 1992
'*    Leap% = LeapYear%(Year%)
'*
'*    Leap% is assigned -1 (TRUE).
'*
'*    Year% = 1900
'*    Leap% = LeapYear%(Year%)
'*
'*    Leap% is assigned 0 (FALSE).
'*
'*    Year% = 2000
'*    Leap% = LeapYear%(Year%)
'*
'*    Leap% is assigned -1 (Year%).
'***********************************************************************
FUNCTION LeapYear% (Year%) STATIC
   LeapYear% = (Year% MOD 4 = 0 AND Year% MOD 100 <> 0) OR (Year% MOD 400 = 0)
END FUNCTION

'***********************************************************************
'* FUNCTION MDY2Date$
'*
'* PURPOSE
'*    Converts Month%, Day%, and Year% to a string in the standard date
'*    format.
'*
'* EXAMPLE
'*    Month% = 10
'*    Day% = 20
'*    Year% = 1995
'*    D$ = MDY2Date$(Month%, Day%, Year%)
'*
'*    D$ is assigned "10-20-1995"
'***********************************************************************
FUNCTION MDY2Date$ (Month%, Day%, Year%) STATIC
   MDY2Date$ = RIGHT$("0" + MID$(STR$(Month%), 2), 2) + "-" + RIGHT$("0" + MID$(STR$(Day%), 2), 2) + "-" + RIGHT$("000" + MID$(STR$(Year%), 2), 4)
END FUNCTION

'***********************************************************************
'* FUNCTION MthName$
'*
'* PURPOSE
'*    Returns the name of the month given a date in the standard date
'*    format.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    M$ = MthName$(DateX$)
'*
'*    M$ is assigned "Oct"
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION MthName$ (DateX$) STATIC
   MthName$ = MID$("JanFebMarAprMayJunJulAugSepOctNovDec", IntVal%(DateX$) * 3 - 2, 3)
END FUNCTION

'***********************************************************************
'* FUNCTION Second2Date$
'*
'* PURPOSE
'*    Returns the date in the standard date format given the seconds
'*    since the last second of 1979.  Use Second2Time$ to find the time
'*    of day at the indicated second.
'*
'* EXAMPLE
'*    Second& = 498614400
'*    D$ = Second2Date$(Second&)
'*
'*    D$ is assigned "10-20-1995"
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION Date2Serial& (DateX$)
'***********************************************************************
FUNCTION Second2Date$ (Second&) STATIC
   Second2Date$ = Serial2Date$(Second& \ 86400 + 2444240)
END FUNCTION

'***********************************************************************
'* FUNCTION Second2Time$
'*
'* PURPOSE
'*    Returns the time in the standard time format given the seconds
'*    since the last second of 1979.  Use Second2Date$ to find the date
'*    at the indicated second.
'*
'* EXAMPLE
'*    Second& = 498614400
'*    T$ = Second2Time$(Second&)
'*
'*    T$ is assigned "00:00:00"
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION HMS2Time$ (Hours%, Minutes%, Seconds%)
'***********************************************************************
FUNCTION Second2Time$ (Second&) STATIC
   Time& = Second& MOD 86400
   Seconds% = Time& MOD 60
   Time& = Time& \ 60
   Minutes% = Time& MOD 60
   Hours% = Time& \ 60
   Second2Time$ = HMS2Time$(Hours%, Minutes%, Seconds%)
END FUNCTION

'***********************************************************************
'* FUNCTION Serial2Date$
'*
'* PURPOSE
'*    Returns a date in the standard date format given a serial date.
'*
'* EXAMPLE
'*    Serial& = 2450011
'*    D$ = Serial2Date$(Serial&)
'*
'*    D$ is assigned "10-20-1995"
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION MDY2Date$ (Month%, Day%, Year%)
'***********************************************************************
FUNCTION Serial2Date$ (Serial&) STATIC
   X& = 4 * Serial& - 6884477
   Y& = (X& \ 146097) * 100
   D& = (X& MOD 146097) \ 4

   X& = 4 * D& + 3
   Y& = (X& \ 1461) + Y&
   D& = (X& MOD 1461) \ 4 + 1

   X& = 5 * D& - 3
   M& = X& \ 153 + 1
   D& = (X& MOD 153) \ 5 + 1

   IF M& < 11 THEN
      Month% = M& + 2
   ELSE
      Month% = M& - 10
   END IF

   Day% = D&
   Year% = Y& + M& \ 11

   DateX$ = MDY2Date$(Month%, Day%, Year%)
   Serial2Date$ = DateX$
END FUNCTION

'***********************************************************************
'* FUNCTION Time2Hour%
'*
'* PURPOSE
'*    Returns the hour number as indicated by a string in the standard
'*    time format.
'*
'* EXAMPLE
'*    TimeX$ = "01:02:00"
'*    H% = Time2Hour%(TimeX$)
'*
'*    H% is assigned 1.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION Time2Hour% (TimeX$) STATIC
   Time2Hour% = IntVal%(TimeX$)
END FUNCTION

'***********************************************************************
'* FUNCTION Time2Meridian$
'*
'* PURPOSE
'*    Appends the meridian to a string in the standard time format and
'*    converts it from 24 hour time.
'*
'* EXAMPLE
'*    TimeX$ = "01:02:00"
'*    T$ = Time2Meridian$(TimeX$)
'*
'*    T$ is assigned " 1:02:00 am".
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION Time2Meridian$ (TimeX$) STATIC
   Time2Meridian$ = SPACE$(11)

   Hour% = IntVal%(TimeX$)
   Meridian$ = MID$(" pm am", 1 - (Hour% < 12) * 3, 3)
   Hour% = Hour% + (Hour% > 12) * 12
   IF Hour% = 0 THEN
      Hour% = 12
   END IF

   TTOMTmp$ = MID$(STR$(Hour%), 2) + RIGHT$(TimeX$, 6) + Meridian$
   RSET Time2Meridian$ = TTOMTmp$

   TTOMTmp$ = ""
   Meridian$ = ""
END FUNCTION

'***********************************************************************
'* FUNCTION Time2Minute%
'*
'* PURPOSE
'*    Returns the minute number as indicated in a string in the standard
'*    time format.
'*
'* EXAMPLE
'*    TimeX$ = "01:02:00"
'*    M% = Time2Minute%(TimeX$)
'*
'*    M% is assigned 2.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION Time2Minute% (TimeX$) STATIC
   Time2Minute% = IntVal%(MID$(TimeX$, 4))
END FUNCTION

'***********************************************************************
'* FUNCTION Time2Second%
'*
'* PURPOSE
'*    Returns the second number as indicated in a string in the standard
'*    time format.
'*
'* EXAMPLE
'*    TimeX$ = "01:02:00"
'*    S% = Time2Second%(TimeX$)
'*
'*    S% is assigned 0.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION Time2Second% (TimeX$) STATIC
   Time2Second% = IntVal%(MID$(TimeX$, 7))
END FUNCTION

'***********************************************************************
'* FUNCTION TimeDate2Second&
'*
'* PURPOSE
'*    Returns the number of seconds since the last second of 1979.  The
'*    date must be between 1980 and 2047.
'*
'* EXAMPLE
'*    TimeX$ = "01:02:00"
'*    DateX$ = "10-20-1995"
'*    S& = TimeDate2Second&(TimeX$, DateX$)
'*
'*    S& is assigned 498,618,120.
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION Date2Serial& (DateX$)
'*    FUNCTION IntVal%()
'***********************************************************************
FUNCTION TimeDate2Second& (TimeX$, DateX$) STATIC
   Days& = Date2Serial&(DateX$) - 2444240
   Hour% = IntVal%(TimeX$)
   Minute% = IntVal%(MID$(TimeX$, 4))
   Second% = IntVal%(RIGHT$(TimeX$, 2))
   Seconds& = CLNG(Hour%) * 3600 + Minute% * 60 + Second%
   TimeDate2Second& = Days& * 86400 + Seconds&
END FUNCTION

'***********************************************************************
'* FUNCTION ValidDate%
'*
'* PURPOSE
'*    Returns TRUE if the given date represents a real date or FALSE if
'*    the date is in error.
'*
'* EXAMPLE
'*    DateX$ = "10-20-1995"
'*    Valid% = ValidDate%(DateX$)
'*
'*    Valid% is assigned -1 (TRUE).
'*
'*    DateX$ = "10-32-1995"
'*    Valid% = ValidDate%(DateX$)
'*
'*    Valid% is assigned 0 (FALSE).
'*
'* INTERNAL ROUTINE(S)
'*    FUNCTION Date2Serial& (DateX$)
'*    FUNCTION Serial2Date$ (Serial&)
'***********************************************************************
FUNCTION ValidDate% (DateX$) STATIC
   ValidDate% = DateX$ = Serial2Date$(Date2Serial&(DateX$))
END FUNCTION

'***********************************************************************
'* FUNCTION WeekDay$
'*
'* PURPOSE
'*    Uses DOS ISR 21H, Function 2AH (Get Date) to return the current
'*    day of the week.
'*
'* EXAMPLE
'*    WkDay$ = WeekDay$
'*
'*    If the date is 10-20-1995, WkDay$ is assigned "Fri".
'*
'* EXTERNAL ROUTINE(S)
'*    QBX.LIB
'*       SUB Interrupt (IntNum%, InReg AS RegType, OutReg AS RegType)
'***********************************************************************
FUNCTION WeekDay$ STATIC
   Regs.ax = &H2A00                          'Get Date
   Interrupt &H21, Regs, Regs                'Call DOS
   WeekDay$ = MID$("SunMonTueWedThuFriSat", ((Regs.ax AND &HFF) + 1) * 3 - 2, 3)
END FUNCTION
